@@ -1,6 +1,6 @@
-pub mod chat;
+pub mod aggregate;
 pub mod compact;
 pub mod models;
 pub mod responses;
 pub mod responses_websocket;
-mod streaming;
+mod session;
