@@ -1,6 +1,17 @@
+use crate::metrics::names::API_CALL_COUNT_METRIC;
+use crate::metrics::names::API_CALL_DURATION_METRIC;
+use crate::metrics::names::SSE_EVENT_COUNT_METRIC;
+use crate::metrics::names::SSE_EVENT_DURATION_METRIC;
+use crate::metrics::names::TOOL_CALL_COUNT_METRIC;
+use crate::metrics::names::TOOL_CALL_DURATION_METRIC;
+use crate::metrics::names::WEBSOCKET_EVENT_COUNT_METRIC;
+use crate::metrics::names::WEBSOCKET_EVENT_DURATION_METRIC;
+use crate::metrics::names::WEBSOCKET_REQUEST_COUNT_METRIC;
+use crate::metrics::names::WEBSOCKET_REQUEST_DURATION_METRIC;
 use crate::otel_provider::traceparent_context_from_env;
 use chrono::SecondsFormat;
 use chrono::Utc;
+use codex_api::ApiError;
 use codex_api::ResponseEvent;
 use codex_app_server_protocol::AuthMode;
 use codex_protocol::ThreadId;
@@ -29,6 +40,9 @@ pub use crate::OtelEventMetadata;
 pub use crate::OtelManager;
 pub use crate::ToolDecisionSource;
 
+const SSE_UNKNOWN_KIND: &str = "unknown";
+const WEBSOCKET_UNKNOWN_KIND: &str = "unknown";
+
 impl OtelManager {
     #[allow(clippy::too_many_arguments)]
     pub fn new(
@@ -148,6 +162,21 @@ impl OtelManager {
         error: Option<&str>,
         duration: Duration,
     ) {
+        let success = status.is_some_and(|code| (200..=299).contains(&code)) && error.is_none();
+        let success_str = if success { "true" } else { "false" };
+        let status_str = status
+            .map(|code| code.to_string())
+            .unwrap_or_else(|| "none".to_string());
+        self.counter(
+            API_CALL_COUNT_METRIC,
+            1,
+            &[("status", status_str.as_str()), ("success", success_str)],
+        );
+        self.record_duration(
+            API_CALL_DURATION_METRIC,
+            duration,
+            &[("status", status_str.as_str()), ("success", success_str)],
+        );
         tracing::event!(
             tracing::Level::INFO,
             event.name = "codex.api_request",
@@ -167,6 +196,134 @@ impl OtelManager {
         );
     }
 
+    pub fn record_websocket_request(&self, duration: Duration, error: Option<&str>) {
+        let success_str = if error.is_none() { "true" } else { "false" };
+        self.counter(
+            WEBSOCKET_REQUEST_COUNT_METRIC,
+            1,
+            &[("success", success_str)],
+        );
+        self.record_duration(
+            WEBSOCKET_REQUEST_DURATION_METRIC,
+            duration,
+            &[("success", success_str)],
+        );
+        tracing::event!(
+            tracing::Level::INFO,
+            event.name = "codex.websocket_request",
+            event.timestamp = %timestamp(),
+            conversation.id = %self.metadata.conversation_id,
+            app.version = %self.metadata.app_version,
+            auth_mode = self.metadata.auth_mode,
+            user.account_id = self.metadata.account_id,
+            user.email = self.metadata.account_email,
+            terminal.type = %self.metadata.terminal_type,
+            model = %self.metadata.model,
+            slug = %self.metadata.slug,
+            duration_ms = %duration.as_millis(),
+            success = success_str,
+            error.message = error,
+        );
+    }
+
+    pub fn record_websocket_event(
+        &self,
+        result: &Result<
+            Option<
+                Result<
+                    tokio_tungstenite::tungstenite::Message,
+                    tokio_tungstenite::tungstenite::Error,
+                >,
+            >,
+            ApiError,
+        >,
+        duration: Duration,
+    ) {
+        let mut kind = None;
+        let mut error_message = None;
+        let mut success = true;
+
+        match result {
+            Ok(Some(Ok(message))) => match message {
+                tokio_tungstenite::tungstenite::Message::Text(text) => {
+                    match serde_json::from_str::<serde_json::Value>(text) {
+                        Ok(value) => {
+                            kind = value
+                                .get("type")
+                                .and_then(|value| value.as_str())
+                                .map(std::string::ToString::to_string);
+                            if kind.as_deref() == Some("response.failed") {
+                                success = false;
+                                error_message = value
+                                    .get("response")
+                                    .and_then(|value| value.get("error"))
+                                    .map(serde_json::Value::to_string)
+                                    .or_else(|| Some("response.failed event received".to_string()));
+                            }
+                        }
+                        Err(err) => {
+                            kind = Some("parse_error".to_string());
+                            error_message = Some(err.to_string());
+                            success = false;
+                        }
+                    }
+                }
+                tokio_tungstenite::tungstenite::Message::Binary(_) => {
+                    success = false;
+                    error_message = Some("unexpected binary websocket event".to_string());
+                }
+                tokio_tungstenite::tungstenite::Message::Ping(_)
+                | tokio_tungstenite::tungstenite::Message::Pong(_) => {
+                    return;
+                }
+                tokio_tungstenite::tungstenite::Message::Close(_) => {
+                    success = false;
+                    error_message =
+                        Some("websocket closed by server before response.completed".to_string());
+                }
+                tokio_tungstenite::tungstenite::Message::Frame(_) => {
+                    success = false;
+                    error_message = Some("unexpected websocket frame".to_string());
+                }
+            },
+            Ok(Some(Err(err))) => {
+                success = false;
+                error_message = Some(err.to_string());
+            }
+            Ok(None) => {
+                success = false;
+                error_message = Some("stream closed before response.completed".to_string());
+            }
+            Err(err) => {
+                success = false;
+                error_message = Some(err.to_string());
+            }
+        }
+
+        let kind_str = kind.as_deref().unwrap_or(WEBSOCKET_UNKNOWN_KIND);
+        let success_str = if success { "true" } else { "false" };
+        let tags = [("kind", kind_str), ("success", success_str)];
+        self.counter(WEBSOCKET_EVENT_COUNT_METRIC, 1, &tags);
+        self.record_duration(WEBSOCKET_EVENT_DURATION_METRIC, duration, &tags);
+        tracing::event!(
+            tracing::Level::INFO,
+            event.name = "codex.websocket_event",
+            event.timestamp = %timestamp(),
+            event.kind = %kind_str,
+            conversation.id = %self.metadata.conversation_id,
+            app.version = %self.metadata.app_version,
+            auth_mode = self.metadata.auth_mode,
+            user.account_id = self.metadata.account_id,
+            user.email = self.metadata.account_email,
+            terminal.type = %self.metadata.terminal_type,
+            model = %self.metadata.model,
+            slug = %self.metadata.slug,
+            duration_ms = %duration.as_millis(),
+            success = success_str,
+            error.message = error_message.as_deref(),
+        );
+    }
+
     pub fn log_sse_event<E>(
         &self,
         response: &Result<Option<Result<StreamEvent, StreamError<E>>>, Elapsed>,
@@ -215,6 +372,16 @@ impl OtelManager {
     }
 
     fn sse_event(&self, kind: &str, duration: Duration) {
+        self.counter(
+            SSE_EVENT_COUNT_METRIC,
+            1,
+            &[("kind", kind), ("success", "true")],
+        );
+        self.record_duration(
+            SSE_EVENT_DURATION_METRIC,
+            duration,
+            &[("kind", kind), ("success", "true")],
+        );
         tracing::event!(
             tracing::Level::INFO,
             event.name = "codex.sse_event",
@@ -236,6 +403,17 @@ impl OtelManager {
     where
         T: Display,
     {
+        let kind_str = kind.map_or(SSE_UNKNOWN_KIND, String::as_str);
+        self.counter(
+            SSE_EVENT_COUNT_METRIC,
+            1,
+            &[("kind", kind_str), ("success", "false")],
+        );
+        self.record_duration(
+            SSE_EVENT_DURATION_METRIC,
+            duration,
+            &[("kind", kind_str), ("success", "false")],
+        );
         match kind {
             Some(kind) => tracing::event!(
                 tracing::Level::INFO,
@@ -443,12 +621,12 @@ impl OtelManager {
     ) {
         let success_str = if success { "true" } else { "false" };
         self.counter(
-            "codex.tool.call",
+            TOOL_CALL_COUNT_METRIC,
             1,
             &[("tool", tool_name), ("success", success_str)],
         );
         self.record_duration(
-            "codex.tool.call.duration_ms",
+            TOOL_CALL_DURATION_METRIC,
             duration,
             &[("tool", tool_name), ("success", success_str)],
         );
